#include <stdio.h>
#define LgMax 400002
#define InFile "secvreg.in"
#define OutFile "secvreg.out"
#define deschis(c)  (c=='(' || c=='[')

/* Numim coada cu dubla prioritate (dequeue) o structura de date care
   suporta urmatoarele operatii:
   - inserare element la sfaristul cozii
   - inserare element la inceputul cozii
   - extragere element de la sfarsitul cozii
   - extragere element de la inceputul cozii
   - accesarea elementului de la inceputul cozii
   - accesarea elementului de la sfarsitul cozii.
  Vom utiliza doua cozi cu dubla prioritate:
  1. Coada a cu elemente de tip caracter in care inseram succesiv caracterele
  din sirul initial si ulterior cele specificate in operatiile din fisierul de intrare.
  2. Coada b cu elemente de tip intreg;
     b[i]=lungimea secventei regulate care incepe cu pozitia i;
  Implementam cozile a si b ca doi vectori.
  Pentru a putea realiza operatii si la inceputul si la sfarsitul vectorilor,
  am dublat spatiul de memorie necesar pentru fiecare dintre cei doi vectori
  si am inceput inserarea elementelor de la mijloc.
  Initial inseram in ordine toate caracterele sirului citit in coada a.
  Pentru a pastra ordinea, fiecare noua caracter va fi inserat la sfarsitul cozii
  Cand la inseram o paranteza deschisa la sfarsitul cozii, iar pe ultima pozitie
  in coada este paranteza deschisa corespunzatoare, deducem ca se formeaza
  o secventa regulata cu lungimea 2 mai mare decat cea deja existenta pe aceasta pozitie.
  Prin urmare retinem in vectorul b aceasta lungime si eliminam din coada a cele doua paranteze
  (practic fiecare secventa regulata formata este eliminata din a, dar in b retinem lungimile secventelor formate).

  Cand inseram la inceputul cozii a o paranteza deschisa iar la inceputul cozii a era paranteza inchisa corespunzatoare,
  se formeaza o secventa regulata cu 2 mai mare si executam aceleasi operatii ca in cazul precedent.
  */

typedef int CoadaI[LgMax];
typedef char CoadaC[LgMax];

char pereche[100];
CoadaC a;
CoadaI b;
/* inca = inceputul cozii a; sfa = sfarsitul cozii a;
   initial coada a este vida */
int inca=LgMax/2, sfa=LgMax/2-1;
/* incb = inceputul cozii b; sfb = sfarsitul cozii b;
   initial coada b este vida */
int incb=LgMax/2, sfb=LgMax/2-1;
char s[LgMax];

int la_sfarsit(char c)
/* functia insereaza caracterul c la sfarsitul cozii a si returneaza lungimea
secventei regulate care contine caracterul inserat */
{ int ret;
  if ( inca<=sfa && !deschis(c) && a[sfa] == pereche[c] )
     {
     ret = b[sfb] + 2;  /* retin lungimea secventei regulate obtinuta */
     sfa--; sfb--; /* elimin secventa regulata formata */
     b[sfb] += ret; /* memorez in b lungimea noi secvente regulate */
     return ret;
     }
  a[++sfa]=c; /*adaug la sfarsitul cozii a caracterul c */
  b[++sfb]=0; /* cu caracterul c se formeaza o secventa regulata de lungime 0 */
  return 0;
}

int la_inceput(char c)
/* functia insereaza caracterul c la inceputul cozii a si returneaza lungimea
secventei regulate care contine caracterul inserat */
{ int ret;
  if (inca<=sfa && deschis(c) && a[inca] == pereche[c] )
     {
     ret = b[incb] + 2; /* retin lungimea secventei regulate obtinuta */
     inca++; incb++;    /* elimin secventa regulata formata */
     b[incb] += ret;    /* memorez in b lungimea noi secvente regulate */
     return ret;
     }
  a[--inca]= c ; /*adaug la inceputul cozii a caracterul c */
  b[--incb]= 0 ;/* cu caracterul c se formeaza o secventa regulata de lungime 0 */
  return 0;
}

int main(void)
{ char * c, ch;
  int n, di, i;
  FILE *fin, *fout;
  fin=fopen(InFile, "rt");
  fscanf(fin,"%s",s);
  fout=fopen(OutFile,"wt");

  pereche['('] = ')'; pereche[')'] = '(';
  pereche['['] = ']'; pereche[']'] = '[';
  /*deocamdata elementele din coada a formeaza o secventa regulata de lungime 0 */
  b[++sfb]=0;
  /* inserez succesiv caracterele din sirul citit la sfarsitul cozii a */
  for (c=s; *c; ++c ) la_sfarsit( *c );
  /* citesc numarul de operatii */
  fscanf(fin, "%d", &n );
  for (i = 0; i < n; i++ )
      {
      /* citesc succesiv operatiile */
      fscanf(fin, "%d %c", &di, &ch );
      if (di) fprintf(fout,"%d\n", la_sfarsit(ch));
         else fprintf(fout,"%d\n", la_inceput(ch));
      }
  fclose(fin);
  fclose(fout);
  return 0;
}
